<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8"/>
  <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
  <title>{{ title }} - {{ SITE_NAME }}</title>

  <script>
    window.data = {{ json_data | tojson }};
  </script>
</head>

<body class="bg-gray-100 min-h-screen h-full min-h-screen">

{{ content|safe }}

</body>
</html>
